package com.processmininguc.slog.parser;

import java.util.ArrayList;
import java.util.List;

import org.antlr.v4.runtime.misc.NotNull;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;


public class SimpleLogRecognizer extends SimpleLogBaseListener{
	
	private static final String LOG_NAME = "SimpleLog Process";
	private static final String TRACE_NAME = "trace";
	
	private XFactory factory;
	private XLog log;
	private List<XTrace> traces;
	private int numTraces;
	
	private long timeCount;
	
	public SimpleLogRecognizer(){
		factory = XFactoryRegistry.instance().currentDefault();
		numTraces = 1;
		timeCount = 946684800000L; //Sat, 01 Jan 2000 00:00:00 GMT
	}
	
	@Override public void enterLog(@NotNull SimpleLogParser.LogContext ctx) {	
		log = factory.createLog();
		log.getExtensions().add(XConceptExtension.instance());
		log.getExtensions().add(XLifecycleExtension.instance());
		log.getExtensions().add(XTimeExtension.instance());
		XConceptExtension.instance().assignName(log, LOG_NAME);
		XLifecycleExtension.instance().assignModel(log, XLifecycleExtension.VALUE_MODEL_STANDARD);
	}

	@Override public void exitLog(@NotNull SimpleLogParser.LogContext ctx) { }
	
	@Override public void enterNumtraces(@NotNull SimpleLogParser.NumtracesContext ctx) {
		numTraces = Integer.parseInt(ctx.getText());
	}

	@Override public void exitNumtraces(@NotNull SimpleLogParser.NumtracesContext ctx) { }
	
	@Override public void enterEvents(@NotNull SimpleLogParser.EventsContext ctx) { 	
		traces = new ArrayList<XTrace>(numTraces);
		for(int i=0; i<numTraces; i++){
			XTrace trace = factory.createTrace();
			traces.add(trace);
			XConceptExtension.instance().assignName(trace, TRACE_NAME+log.size());
			log.add(trace);
		}
	}

	@Override public void exitEvents(@NotNull SimpleLogParser.EventsContext ctx) {
		//Consecutive (and non overlapping) time of events
		for(XTrace trace: traces){
			for(int i=0; i<trace.size(); i++){
				XEvent event = trace.get(i);
				XAttributeMap attributes = event.getAttributes();
				attributes.put(XTimeExtension.KEY_TIMESTAMP,
						factory.createAttributeTimestamp(XTimeExtension.KEY_TIMESTAMP, timeCount, XTimeExtension.instance()));
				//XTimeExtension.instance().assignTimestamp(event, timeCount); //Buffered attributes need to be re-set to be permanent
				timeCount += 1000;
				event.setAttributes(attributes);
				trace.set(i, event); //Buffered traces need to be re-set to be permanent
			}
		}
		//Re-set number of traces
		numTraces = 1;
	}

	@Override public void enterTrace(@NotNull SimpleLogParser.TraceContext ctx) {
		
	}

	@Override public void exitTrace(@NotNull SimpleLogParser.TraceContext ctx) { 

	}
	
	@Override public void enterEvent(@NotNull SimpleLogParser.EventContext ctx) { 	
		
		for(XTrace trace: traces){
			XEvent event = factory.createEvent();
			XConceptExtension.instance().assignName(event, ctx.getText());
			XConceptExtension.instance().assignInstance(event, ctx.getText());
			XLifecycleExtension.instance().assignStandardTransition(event, StandardModel.COMPLETE);
			trace.add(event);
		}
	}

	@Override public void exitEvent(@NotNull SimpleLogParser.EventContext ctx) { }

	
	public XLog getLog(){
		return this.log;
	}

}
