package com.processmininguc.srlog.parser;

import java.util.ArrayList;
import java.util.List;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension.StandardModel;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;

public class SimpleResourceLogRecognizer extends SimpleResourceLogBaseListener{
	
	
	private static final String LOG_NAME = "SimpleCostLog Process";
	private static final String TRACE_NAME = "trace";
	
	private XFactory factory;
	private XLog log;
	private List<XTrace> traces;
	private int numTraces;
	private String activity;
	private String resource;
	
	private long timeCount;
	
	public SimpleResourceLogRecognizer(){
		factory = XFactoryRegistry.instance().currentDefault();
		numTraces = 1;
		timeCount = 946684800000L; //Sat, 01 Jan 2000 00:00:00 GMT
	}
	
	
	@Override public void enterLog(SimpleResourceLogParser.LogContext ctx) {
		log = factory.createLog();
		log.getExtensions().add(XConceptExtension.instance());
		log.getExtensions().add(XLifecycleExtension.instance());
		log.getExtensions().add(XTimeExtension.instance());
		log.getExtensions().add(XOrganizationalExtension.instance());
		XConceptExtension.instance().assignName(log, LOG_NAME);
		XLifecycleExtension.instance().assignModel(log, XLifecycleExtension.VALUE_MODEL_STANDARD);
	}

	@Override public void exitLog(SimpleResourceLogParser.LogContext ctx) { }

	
	@Override public void enterTrace(SimpleResourceLogParser.TraceContext ctx) { }

	@Override public void exitTrace(SimpleResourceLogParser.TraceContext ctx) { }
	

	@Override public void enterEvents(SimpleResourceLogParser.EventsContext ctx) {
		traces = new ArrayList<XTrace>(numTraces);
		for(int i=0; i<numTraces; i++){
			XTrace trace = factory.createTrace();
			traces.add(trace);
			XConceptExtension.instance().assignName(trace, TRACE_NAME+log.size());
			log.add(trace);
		}
	}

	@Override public void exitEvents(SimpleResourceLogParser.EventsContext ctx) {
		//Consecutive (and non overlapping) time of events
		for(XTrace trace: traces){
			for(int i=0; i<trace.size(); i++){
				XEvent event = trace.get(i);
				XAttributeMap attributes = event.getAttributes();
				attributes.put(XTimeExtension.KEY_TIMESTAMP,
						factory.createAttributeTimestamp(XTimeExtension.KEY_TIMESTAMP, timeCount, XTimeExtension.instance()));
				//XTimeExtension.instance().assignTimestamp(event, timeCount); //Buffered attributes need to be re-set to be permanent
				timeCount += 1000;
				event.setAttributes(attributes);
				trace.set(i, event); //Buffered traces need to be re-set to be permanent
			}
		}
		//Re-set number of traces
		numTraces = 1;
	}

	@Override public void enterNumtraces(SimpleResourceLogParser.NumtracesContext ctx) {
		numTraces = Integer.parseInt(ctx.getText());
	}

	@Override public void exitNumtraces(SimpleResourceLogParser.NumtracesContext ctx) { }

	@Override public void enterEvent(SimpleResourceLogParser.EventContext ctx) { }

	@Override public void exitEvent(SimpleResourceLogParser.EventContext ctx) {
		for(XTrace trace: traces){
			XEvent event = factory.createEvent();
			XConceptExtension.instance().assignName(event, activity);
			XConceptExtension.instance().assignInstance(event, activity);
			XLifecycleExtension.instance().assignStandardTransition(event, StandardModel.COMPLETE);
			XOrganizationalExtension.instance().assignResource(event, resource);
			
			trace.add(event);
		}
	}

	@Override public void enterActivity(SimpleResourceLogParser.ActivityContext ctx) {
		activity = ctx.getText();
	}

	@Override public void exitActivity(SimpleResourceLogParser.ActivityContext ctx) { }

	@Override public void enterResource(SimpleResourceLogParser.ResourceContext ctx) {
		resource = ctx.getText();
	}

	@Override public void exitResource(SimpleResourceLogParser.ResourceContext ctx) { }

	public XLog getLog(){
		return this.log;
	}

}
